﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Globalization;


namespace MindFusion.Scheduling.Wpf.Samples.CS.ResourceTimeline
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			// Create some test resources
			Appointment app = null;
			Random r = new Random();

			_calendar.ItemResources.Add(new Resource { Name = "Resource #1" });
			_calendar.ItemResources.Add(new Resource { Name = "Resource #2" });
			_calendar.ItemResources.Add(new Resource { Name = "Resource #3" });
			_calendar.GroupType = GroupType.GroupByResources;

			for (int i = 0; i < 20; i++)
			{
				app = new Appointment();
				app.StartTime = _calendar.Date + TimeSpan.FromDays(r.Next(10) + 3);
				app.EndTime = app.StartTime + TimeSpan.FromHours(24 + r.Next(24));
				app.HeaderText = "Resource " + i.ToString();
				app.Resources.Add(_calendar.ItemResources[r.Next(3)]);
				_calendar.Schedule.Items.Add(app);
			}

			_calendar.HScroll += (s, e) =>
			{
				if (e.ScrollEventType == ScrollEventType.EndScroll)
					UpdateTimeline();
			};

			_calendar.Rebuilt += (s, e) =>
			{
				InitTimeline();
				UpdateTimeline();
				_calendar.SetViewport(new DateTime((long)timeline.SelectionStart),
					new DateTime((long)timeline.SelectionEnd), true);
			};

			timeline.ValueChanged += (s, e) =>
			{
				_calendar.SetViewport(new DateTime((long)timeline.SelectionStart),
					new DateTime((long)timeline.SelectionEnd), true);
			};

			timeline.DrawBackground += (s, e) =>
			{
				var bounds = new Rect(0, 0, e.Control.ActualWidth, e.Control.ActualHeight);

				// Draw days
				var date = _calendar.Date;
				while (date < _calendar.EndDate)
				{
					double prc = (double)(date.Ticks - _calendar.Date.Ticks) / (_calendar.EndDate.Ticks - _calendar.Date.Ticks);

					var x = bounds.X + prc * bounds.Width;
					var pen = new Pen(timeline.BorderBrush, 1);
					e.Context.DrawLine(pen, new Point(x, bounds.Top), new Point(x, bounds.Bottom));

					var text = new FormattedText(date.ToString("MMM"), CultureInfo.InvariantCulture,
						FlowDirection.LeftToRight, new Typeface(_calendar.ResourceViewSettings.HeaderStyle.FontFamily,
							_calendar.ResourceViewSettings.CalendarStyle.FontStyle,
							_calendar.ResourceViewSettings.CalendarStyle.FontWeight,
							_calendar.ResourceViewSettings.CalendarStyle.FontStretch),
						_calendar.ResourceViewSettings.HeaderStyle.FontSize, _calendar.ResourceViewSettings.HeaderStyle.Foreground);
					e.Context.DrawText(text, new Point(x, bounds.Top));

					date = date.AddDays(DateTime.DaysInMonth(date.Year, date.Month) - date.Day + 1);
				}
			};

			SizeChanged += (s, e) =>
			{
				UpdateTimeline();
			};
		}

		/// <summary>
		/// Initialize the timeline from the Calendar control.
		/// </summary>
		void InitTimeline()
		{
			DateTime start = _calendar.Date;
			DateTime end = _calendar.EndDate;

			timeline.Minimum = start.Ticks;
			timeline.Maximum = end.Ticks;

			timeline.MinimumSelectionLength = (timeline.Maximum - timeline.Minimum) / 30;
			timeline.Step = TimeSpan.TicksPerDay;
			timeline.SelectionStart = timeline.Minimum;
			timeline.SelectionEnd = timeline.SelectionStart + timeline.MinimumSelectionLength;
		}

		/// <summary>
		/// Updates the timeline from the Calendar control.
		/// </summary>
		void UpdateTimeline()
		{
			double newStart = _calendar.GetFirstVisibleDate().Ticks;
			double newEnd = _calendar.GetLastVisibleDate().Ticks;

			if (newStart < timeline.Minimum)
				newStart = timeline.Minimum;
			if (newEnd > timeline.Maximum)
				newEnd = timeline.Maximum;

			timeline.SelectionStart = newStart;
			timeline.SelectionEnd = newEnd;
		}
	}
}
